import http from 'http'
import { URL } from 'url'
import { defaults } from './helper.mjs'

export default function Client(port, opts) {
  this.options = defaults(opts, {})
  this.prefix = `http://localhost:${port}`
}

Client.prototype.customRequest = function(method = 'GET', path, body, options) {
  if (path.slice(0, 4) !== 'http') {
    path = this.prefix + path
  }
  let urlObj = new URL(path)

  return new Promise((resolve, reject) => {
    const opts = defaults(defaults(options, {
      agent: false,
      method: method,
      timeout: 500,
      protocol: urlObj.protocol,
      username: urlObj.username,
      password: urlObj.password,
      host: urlObj.hostname,
      port: Number(urlObj.port),
      path: urlObj.pathname + urlObj.search,
    }))

    const req = http.request(opts)
    if (body) {
      req.write(body)
    }

    req.on('error', reject)
    req.on('timeout', function() {
      console.log(req.destroy())
      reject(new Error(`Request ${method} ${path} timed out`))
    })
    req.on('response', res => {
      res.setEncoding('utf8')
      let output = ''

      res.on('data', function (chunk) {
        output += chunk.toString()
      })

      res.on('end', function () {
        try {
          output = JSON.parse(output)
        } catch (e) {
          return reject(new Error(`${e.message} while decoding: ${output}`))
        }
        if (output.status && typeof(output.status) === 'number') {
          let err = new Error(`Request failed [${output.status}]: ${output.message}`)
          err.body = output
          return reject(err)
        }
        resolve(output)
      })
    })
    req.end()
    return req
  })
}

Client.prototype.get = function(path = '/') {
  return this.customRequest('GET', path, null)
}